#ifndef __VFD_M18ST05B_H__
#define __VFD_M18ST05B_H__

#include <stdio.h>
#include <inttypes.h>
#include <avr/io.h>
#include <avr/pgmspace.h>

/* adapt to your U(S)ART implementation:
 *   - uart_putc(char c)
 *   - uart_puts(char *str)
 * 	 - uart_puts_P(const char *str)
 * 
 * uart_puts    sends a zero terminated string to U(S)ART, ends with \0.
 * uart_putc    must be able to send every byte, including \0.
 * 
 * see example main.c
 * 
 */

#define vfd_putc			uart_putc
#define vfd_puts			uart_puts

void vfd_cmd(char *cmd);
void vfd_cmd_P(const char *cmd);
void vfd_puts_P(const char *str);

extern void vfd_putc(char c);
extern void vfd_puts(char *str);


/* WARNING
 * if you want to reuse these sequences, omit first byte. this byte
 * encodes command length for vfd_cmd() and vfd_cmd_P() functions.
 */

/* commands */
#define VFD_CLEAR_AND_LBREAK 	"\x01\x1F"							// enables line break after 16 chars
#define VFD_CLEAR_AND_LBREAK2	"\x02\x1B\x50"						// enables line break after 16 chars
#define VFD_CLEAR_TEXT			"\x06\x1B\x22\x1BP\x1B\x21\x1BP"	// sequence composed of GOTO_LINE and CLEAR_CURRENT
																	// commands. This keeps default reset behaviour: 
																	// scrolling instead of line breaking
#define VFD_CLEAR_CURRENT		"\x02\x1BP"

#define VFD_LINE_ENABLE		"\x02\x1BR"
#define VFD_LINE_DISABLE	"\x02\x1BS"

#define VFD_GOTO_LINE		"\x02\x1B\x20"	// ??
#define VFD_GOTO_LINE0		"\x02\x1B\x21"
#define VFD_GOTO_LINE1		"\x02\x1B\x22"
#define VFD_CR				"\x02\x1BQ"		// moves cursor to first column in current row
#define VFD_CURSOR_HOME		"\x02\x1B\x51"	// same as VFD_CR

/* display test commands */
// ?? still unknown how to end test modes. Once one of these is
// active, display only accepts a few codes.
#define VFD_TEST1	"\x02\x1B\xF0"
#define VFD_TEST2	"\x02\x1B\xF1"

/* display dimming */
#define VFD_DIM0	"\x03\x1B\x40\x00"
#define VFD_DIM1	"\x03\x1B\x40\x01"
#define VFD_DIM2	"\x03\x1B\x40\x02"
#define VFD_DIM3	"\x03\x1B\x40\x03"
#define VFD_DIM4	"\x03\x1B\x40\x04"
#define VFD_DIM5	"\x03\x1B\x40\x05"

/* 
 * rainy bad weather animation across whole VFD 
 * 
 * - display dimming is not accepted anymore when running animation
 * - send VFD_CLEAR_AND_LBREAK to quit rain animation.
 * 
 * */
#define VFD_RAIN_ANIMATION		"\x02\x1B\x54"

/* VFD's MCU keeps a clock running, related stuff: 
 * 
 * - display dimming is accepted while clock is running
 * - send anything (text or commands) to quit clock
 * - if clock is not set, it will show but not run
 * 
 * example how to set clock:
 *
 * 	// this sets clock to 20:48 on 15th November 2019
 *	vfd_cmd("\x08\x1B\x00\x48\x20\x15\x11\x20\x19");
 *
 * */
#define VFD_CLOCK_SHOW			"\x02\x1B\x05"
#define VFD_CLOCK_MOVING		"\x02\x1B\x04"
#define VFD_CLOCK_NOT_MOVING	"\x02\x1B\x03"
#define VFD_CLOCK_SET			"\x02\x1B\x00"

/* VOLUME area of VFD */
#define VFD_FRAME_VOLUME		"\x04\x1B\x30\x1C\x03"
#define VFD_FRAME_VOLUME_OFF	"\x04\x1B\x30\x1C\x00"

#define VFD_VOLUME1				"\x04\x1B\x30\x0B\x03"
#define VFD_VOLUME2				"\x04\x1B\x30\x0C\x03"
#define VFD_VOLUME3				"\x04\x1B\x30\x0D\x03"
#define VFD_VOLUME4				"\x04\x1B\x30\x0E\x03"
#define VFD_VOLUME5				"\x04\x1B\x30\x0F\x03"
#define VFD_VOLUME6				"\x04\x1B\x30\x10\x03"
#define VFD_VOLUME7				"\x04\x1B\x30\x11\x03"

#define VFD_VOLUME1_OFF			"\x04\x1B\x30\x0B\x00"
#define VFD_VOLUME2_OFF			"\x04\x1B\x30\x0C\x00"
#define VFD_VOLUME3_OFF			"\x04\x1B\x30\x0D\x00"
#define VFD_VOLUME4_OFF			"\x04\x1B\x30\x0E\x00"
#define VFD_VOLUME5_OFF			"\x04\x1B\x30\x0F\x00"
#define VFD_VOLUME6_OFF			"\x04\x1B\x30\x10\x00"
#define VFD_VOLUME7_OFF			"\x04\x1B\x30\x11\x00"

#define VFD_VOLUME_RED_LINE 	"\x04\x1B\x30\x12\x03"
#define VFD_VOLUME_RED_LINE_OFF	"\x04\x1B\x30\x12\x00"

#define VFD_UNMUTE				"\x04\x1B\x30\x13\x03"
#define VFD_UNMUTE_OFF			"\x04\x1B\x30\x13\x00"

#define VFD_MUTE				"\x04\x1B\x30\x14\x03"
#define VFD_MUTE_OFF			"\x04\x1B\x30\x14\x00"

#define VFD_VOLUME_SMALL1		"\x04\x1B\x30\x15\x03"
#define VFD_VOLUME_SMALL1_OFF	"\x04\x1B\x30\x15\x00"
#define VFD_VOLUME_SMALL2		"\x04\x1B\x30\x16\x03"
#define VFD_VOLUME_SMALL2_OFF	"\x04\x1B\x30\x16\x00"
#define VFD_VOLUME_SMALL3		"\x04\x1B\x30\x17\x03"
#define VFD_VOLUME_SMALL3_OFF	"\x04\x1B\x30\x17\x00"
#define VFD_SMALL_VOLUME_INDICATOR_OFF	"\x03\x1B\x34\x00"

/* SOURCE AREA of VFD */
#define VFD_FRAME_SOURCE		"\x04\x1B\x30\x18\x03"
#define VFD_FRAME_SOURCE_OFF	"\x04\x1B\x30\x18\x00"

#define VFD_SYMBOL_HDD_OFF  	"\x04\x1b\x30\x00\x00"
#define VFD_SYMBOL_HDD_DIM1 	"\x04\x1b\x30\x00\x01"
#define VFD_SYMBOL_HDD_DIM2 	"\x04\x1b\x30\x00\x02"
#define VFD_SYMBOL_HDD_DIM3 	"\x04\x1b\x30\x00\x03"

#define VFD_SYMBOL_1394_OFF  	"\x04\x1b\x30\x01\x00"
#define VFD_SYMBOL_1394_DIM1 	"\x04\x1b\x30\x01\x01"
#define VFD_SYMBOL_1394_DIM2 	"\x04\x1b\x30\x01\x02"
#define VFD_SYMBOL_1394_DIM3 	"\x04\x1b\x30\x01\x03"

#define VFD_SYMBOL_CD_OFF  		"\x04\x1b\x30\x02\x00"
#define VFD_SYMBOL_CD_DIM1 		"\x04\x1b\x30\x02\x01"
#define VFD_SYMBOL_CD_DIM2 		"\x04\x1b\x30\x02\x02"
#define VFD_SYMBOL_CD_DIM3 		"\x04\x1b\x30\x02\x03"
#define VFD_CD_ROTATE0			"\x03\x1B\x32\x00"	// 0 = not rotating, 1 fastest, 4 slowest
#define VFD_CD_ROTATE1			"\x03\x1B\x32\x01"
#define VFD_CD_ROTATE2			"\x03\x1B\x32\x02"
#define VFD_CD_ROTATE3			"\x03\x1B\x32\x03"
#define VFD_CD_ROTATE4			"\x03\x1B\x32\x04"

#define VFD_SYMBOL_USB_OFF  	"\x04\x1b\x30\x03\x00"
#define VFD_SYMBOL_USB_DIM1 	"\x04\x1b\x30\x03\x01"
#define VFD_SYMBOL_USB_DIM2 	"\x04\x1b\x30\x03\x02"
#define VFD_SYMBOL_USB_DIM3 	"\x04\x1b\x30\x03\x03"

/* MEDIA AREA of VFD */
#define VFD_FRAME_MEDIA			"\x04\x1B\x30\x19\x03"
#define VFD_FRAME_MEDIA_OFF		"\x04\x1B\x30\x19\x00"

#define VFD_SYMBOL_MOVIE_OFF  	"\x04\x1b\x30\x04\x00"
#define VFD_SYMBOL_MOVIE_DIM1 	"\x04\x1b\x30\x04\x01"
#define VFD_SYMBOL_MOVIE_DIM2 	"\x04\x1b\x30\x04\x02"
#define VFD_SYMBOL_MOVIE_DIM3 	"\x04\x1b\x30\x04\x03"

#define VFD_SYMBOL_TV_OFF  		"\x04\x1b\x30\x05\x00"
#define VFD_SYMBOL_TV_DIM1 		"\x04\x1b\x30\x05\x01"
#define VFD_SYMBOL_TV_DIM2 		"\x04\x1b\x30\x05\x02"
#define VFD_SYMBOL_TV_DIM3 		"\x04\x1b\x30\x05\x03"

#define VFD_SYMBOL_MUSIC_OFF  	"\x04\x1b\x30\x06\x00"
#define VFD_SYMBOL_MUSIC_DIM1 	"\x04\x1b\x30\x06\x01"
#define VFD_SYMBOL_MUSIC_DIM2 	"\x04\x1b\x30\x06\x02"
#define VFD_SYMBOL_MUSIC_DIM3 	"\x04\x1b\x30\x06\x03"

#define VFD_SYMBOL_PICTURE_OFF  "\x04\x1b\x30\x07\x00"
#define VFD_SYMBOL_PICTURE_DIM1 "\x04\x1b\x30\x07\x01"
#define VFD_SYMBOL_PICTURE_DIM2 "\x04\x1b\x30\x07\x02"
#define VFD_SYMBOL_PICTURE_DIM3 "\x04\x1b\x30\x07\x03"

/* LETTER AREA of VFD */
#define VFD_FRAME_LETTER			"\x04\x1B\x30\x1B\x03"
#define VFD_FRAME_LETTER_OFF		"\x04\x1B\x30\x1B\x00"

#define VFD_SYMBOL_LETTER_INNER		"\x04\x1B\x30\x0A\x03"
#define VFD_SYMBOL_LETTER_INNER_OFF	"\x04\x1B\x30\x0A\x00"

#define VFD_SYMBOL_LETTER_OUTER		"\x04\x1B\x30\x09\x03"
#define VFD_SYMBOL_LETTER_OUTER_OFF	"\x04\x1B\x30\x09\x00"

/* RECORD AREA of VFD */
#define VFD_FRAME_REC			"\x04\x1B\x30\x1A\x03"
#define VFD_FRAME_REC_OFF		"\x04\x1B\x30\x1A\x00"

#define VFD_SYMBOL_REC			"\x04\x1b\x30\x08\x03"
#define VFD_SYMBOL_REC_OFF		"\x04\x1b\x30\x08\x00"

#define VFD_REC_BLINK0			"\x03\x1B\x33\x00"	// 0 = not rotating, 1 fastest, 4 slowest
#define VFD_REC_BLINK1			"\x03\x1B\x33\x01"
#define VFD_REC_BLINK2			"\x03\x1B\x33\x02"
#define VFD_REC_BLINK3			"\x03\x1B\x33\x03"
#define VFD_REC_BLINK4			"\x03\x1B\x33\x04"

#define VFD_REC_GFXDATA_HEADER	"\x02\x1B\x31"		// followed by 9 bytes gfx data

#endif // __VFD_M18ST05B_H__
